@@ -3,10 +3,10 @@
 #
 # This program is free software with ABSOLUTELY NO WARRANTY.
 # You can re-distribute and/or modify this program under
-# the same terms of the Ruby's lisence.
+# the same terms of the Ruby's license.
 #
 #
-# $Id: server.rb 173 2006-10-02 12:45:38Z ko1 $
+# $Id: server.rb 206 2009-08-21 06:50:20Z znz $
 # Create : K.S. 04/04/17 17:00:44
 #
 
@@ -26,16 +26,20 @@ module Nadoka
     
     def initialize rc
       @rc = rc
-      @clients= []
+      @clients = []
       @prev_timer = Time.now
 
+      @server_thread = nil
+      @clients_thread = nil
+
       @state  = nil
 
       @state  = NDK_State.new self
       reload_config
       
       @server = nil
-      
+      @cserver = nil
+
       @connected = false
       @exitting  = false
 
@@ -53,7 +57,7 @@ module Nadoka
     def next_server_info
       svinfo = @config.server_list.sort_by{rand}.shift
       @config.server_list.push svinfo
-      [svinfo[:host], svinfo[:port], svinfo[:pass]]
+      [svinfo[:host], svinfo[:port], svinfo[:pass], svinfo[:ssl_params]]
     end
     
     def reload_config
@@ -110,8 +114,8 @@ module Nadoka
     end
     
     def make_server
-      host, port, @server_passwd = next_server_info
-      server = ::RICE::Connection.new(host, port)
+      host, port, @server_passwd, ssl_params = next_server_info
+      server = ::RICE::Connection.new(host, port, "\r\n", ssl_params)
       server.regist{|rq, wq|
         Thread.stop
         @rq = rq
@@ -198,7 +202,9 @@ module Nadoka
       if @clients.size == 0
         enter_away
       end
-      
+
+      invoke_event :invoke_bot, :server_connected
+
       # loop
       while q = recv_from_server
         
@@ -300,6 +306,7 @@ module Nadoka
     end
     
     def start_clients_thread
+      return unless @config.client_server_port
       @clients_thread = Thread.new{
         begin
           @cserver = TCPServer.new(@config.client_server_host,
@@ -411,6 +418,10 @@ module Nadoka
         end
         
         # Server -> Nadoka message
+        if !@config.primitive_filters.nil? && !@config.primitive_filters[q.command].nil? && !@config.primitive_filters[q.command].empty?
+          next unless filter_message(@config.primitive_filters[q.command], q)
+        end
+
         case q.command
         when 'PING'
           @server << Cmd.pong(q.params[0])
@@ -607,7 +618,7 @@ module Nadoka
     end
     
     def ctcp_message? arg
-      arg[0] == 1
+      arg[0] == ?\x1
     end
     
     def ctcp_message msg
