@@ -1,3 +1,4 @@
+# coding: ASCII-8BIT
 =begin
 
 = rice - Ruby Irc interfaCE
@@ -12,13 +13,18 @@ Original Credit:
 == Modified
 
   Modified by K.Sasada.
-  $Id: irc.rb 167 2006-03-27 10:57:33Z ko1 $
+  $Id: irc.rb 204 2009-08-20 16:45:18Z znz $
   
 =end
 
 require 'socket'
 require 'thread'
 require 'monitor'
+begin
+  require "openssl"
+rescue LoadError
+end
+
 
 module RICE
   class Error < StandardError; end
@@ -41,7 +47,7 @@ module RICE
 
 =end
 
-    def initialize(server, port, eol = "\r\n")
+    def initialize(server, port, eol = "\r\n", ssl_params = nil)
       @conn = []
       @conn.extend(MonitorMixin)
       @main_th = nil
@@ -49,6 +55,7 @@ module RICE
       self.server = server
       self.port   = port
       self.eol    = eol
+      self.ssl_params = ssl_params
 
       @read_q  = Queue.new
 
@@ -81,7 +88,7 @@ module RICE
       @prev_send_time = Time.now
     end
     attr :delay, true
-    attr_reader :server, :port
+    attr_reader :server, :port, :ssl_params
     
 =begin
 
@@ -121,6 +128,46 @@ module RICE
 
 =begin
 
+--- RICE::Connection#ssl_params=(ssl_params)
+
+=end
+
+    def ssl_params=(ssl_params)
+      raise RuntimeError, 
+        "Already connected to #{@server}:#{@port}" unless @conn.empty?
+      unless ssl_params
+        @ssl_params = false
+        return
+      end
+      raise 'openssl library not installed' unless defined?(OpenSSL)
+      ssl_params = ssl_params.to_hash
+      ssl_params[:verify_mode] ||= OpenSSL::SSL::VERIFY_PEER
+      store = OpenSSL::X509::Store.new
+      if ssl_params.key?(:ca_cert)
+        ca_cert = ssl_params.delete(:ca_cert)
+        if ca_cert
+          # auto setting ca_path or ca_file like open-uri.rb
+          if File.directory? ca_cert
+            store.add_path ca_cert
+          else
+            store.add_file ca_cert
+          end
+        else
+          # use ssl_params={:ca_cert=>nil} if you want to disable auto setting
+          store = nil
+        end
+      else
+        # use default of openssl
+        store.set_default_paths
+      end
+      if store
+        ssl_params[:cert_store] = store
+      end
+      @ssl_params = ssl_params
+    end
+
+=begin
+
 --- RICE::Connection#start(max_retry = 3, retry_wait = 30)
 
 =end
@@ -163,7 +210,18 @@ module RICE
 
     def open_conn
       @conn.synchronize do
-        @conn[0] = TCPSocket.new(@server, @port)
+        conn = TCPSocket.new(@server, @port)
+        if ssl_params
+          context = OpenSSL::SSL::SSLContext.new
+          context.set_params(ssl_params)
+          conn = OpenSSL::SSL::SSLSocket.new(conn, context)
+          conn.sync_close = true
+          conn.connect
+          if context.verify_mode != OpenSSL::SSL::VERIFY_NONE
+            conn.post_connection_check(@server)
+          end
+        end
+        @conn[0] = conn
       end
       @conn[0].extend(MonitorMixin)
 
@@ -305,8 +363,9 @@ module RICE
 =end
 
     def push(message)
-      if @conn[0]
-        @conn[0].synchronize do
+      conn = @conn[0]
+      if conn
+        conn.synchronize do
           cmd = message.command
           if cmd == 'PRIVMSG' || cmd == 'NOTICE'
             # flood control
@@ -316,7 +375,7 @@ module RICE
             end
             @prev_send_time = t
           end
-          @conn[0].print message.to_s unless @conn[0].closed?
+          conn.print message.to_s unless conn.closed?
         end
       else
         nil
@@ -348,7 +407,7 @@ module RICE
       #                 ; as specified in RFC 1123 [HNAME]
       # hostname   =  shortname *( "." shortname )
       SHORTNAME = "[#{LETTER}#{DIGIT}](?:[-#{LETTER}#{DIGIT}\/]*[#{LETTER}#{DIGIT}])?"
-      HOSTNAME  = "#{SHORTNAME}(?:\\.#{SHORTNAME})*"
+      HOSTNAME  = "#{SHORTNAME}(?:\\.#{SHORTNAME})*\\.?"
       
       # servername =  hostname
       SERVERNAME = HOSTNAME
